from decimal import Decimal
from datetime import datetime, date
from django.utils.functional import Promise

from ua2.MongoLog.document import LogDocument


class Logger(object):
    INFO = 'INFO'
    ERROR = 'ERROR'
    DEBUG = 'DEBUG'
    NOTICE = 'NOTICE'
    WARNING = 'WARNING'

    def __init__(self, doc_type):
        self.doc_type = doc_type

    def log(self, level, key, descr, **kwargs):
        log_document = self.doc_type()
        log_document.level = level
        log_document.key = key

        for key, value in kwargs.iteritems():
            if isinstance(value, (Decimal, datetime, date, Promise)):
                kwargs[key] = unicode(value)

        for key, item in kwargs.iteritems():
            setattr(log_document, key, item)

        while True:
            try:
                log_document.descr = descr.format(**kwargs)
                break
            except KeyError, e:
                kwargs[e.args[0]] = '<None>'

        log_document.save(safe=False, validate=False)

    @property
    def objects(self):
        return self.doc_type.objects

    def info(self, *args, **kwargs):
        return self.log(Logger.INFO, *args, **kwargs)

    def error(self, *args, **kwargs):
        return self.log(Logger.ERROR, *args, **kwargs)

    def debug(self, *args, **kwargs):
        return self.log(Logger.DEBUG, *args, **kwargs)

    def notice(self, *args, **kwargs):
        return self.log(Logger.NOTICE, *args, **kwargs)

    def warning(self, *args, **kwargs):
        return self.log(Logger.WARNING, *args, **kwargs)

    def iter_log(self, wrap_cls, qs):
        wrapper = wrap_cls()

        for item in qs.ensure_index('stamp'):
            wrapper.data = item._data
            yield wrapper


log = Logger(LogDocument)



from datetime import datetime
import mongoengine
import pprint
from django.utils.encoding import force_unicode


class LogDocument(mongoengine.DynamicDocument):
    stamp = mongoengine.DateTimeField(required=True, default=datetime.now)
    descr = mongoengine.StringField()
    key = mongoengine.StringField(max_length=255, required=False)
    level = mongoengine.StringField(max_length=255, required=False)

    meta = {'collection': 'log'}
    

    def __unicode__(self):
        return pprint.pformat(self._data)

    def __str__(self):
        return force_unicode(self).encode('utf-8')




#!/usr/bin/env python
try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages


Name='ua2.mongolog'
ProjecUrl="https://bitbucket.org/ua2web/ua2.mongolog"
Version='0.1.2'
Author='Vic'
AuthorEmail='vic@ua2web.com'
Maintainer='Vic'
Summary='Feed logging into MongoDB'
License='BSD License'
ShortDescription=Summary
Description=Summary

needed = [
    'mongoengine>=0.7.5',
]

EagerResources = [
    'ua2',
]

ProjectScripts = [
##    'scripts/runweb',
]

PackageData = {
    '': ['*.*'],
}

# Make exe versions of the scripts:
EntryPoints = {
}

setup(
#    url=ProjecUrl,
    name=Name,
    zip_safe=False,
    version=Version,
    author=Author,
    author_email=AuthorEmail,
    description=ShortDescription,
    long_description=Description,
    license=License,
    scripts=ProjectScripts,
    install_requires=needed,
    include_package_data=True,
    packages=find_packages('src'),
    package_data=PackageData,
    package_dir = {'': 'src'},
    eager_resources = EagerResources,
    entry_points = EntryPoints,
    namespace_packages = ['ua2'],
)
